attribute vec4 pos;
attribute vec4 tCoordinate;
attribute vec3 normal;
uniform mat4 model;
uniform mat4 view;
uniform mat4 project;
uniform mat4 tMat;
uniform int tType;
varying vec3 vCoordinate;
varying vec3 vFragPos;
varying vec3 vNormal;
void main() {
    gl_Position = project * view * model * pos;
    vFragPos = vec3(model * pos);
    vNormal =  mat3(model) * normal;
    if (tType==2)
    {
        vCoordinate =  pos.xyz;
    } else {
        vCoordinate = (tMat * tCoordinate).xyz;
    }
}//@@
    #extension GL_OES_EGL_image_external : require
precision mediump float;
struct Light{
    vec3 pos;
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
};
struct Material{
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
    float shininess;
    int type;
};
varying vec3 vCoordinate;
varying vec3 vFragPos;
varying vec3 vNormal;

uniform samplerCube texture3d;
uniform sampler2D texture2d;
uniform sampler2D texture2dOes;
uniform Light light;
uniform Material material;
uniform vec3 viewPos;
uniform float alpha;
uniform int enableLight;
uniform sampler2D mask;
uniform int maskType;
void main() {
    vec4 img;
    if (material.type == 2){
        img = textureCube(texture3d, vCoordinate);
    } else if (material.type == 3){
        img = texture2D(texture2dOes, vCoordinate.xy);
    } else if (material.type == 1) {
        img = texture2D(texture2d, vCoordinate.xy);
    } else {
        img = vec4(material.diffuse + material.ambient, 1.0);
    }
    if (enableLight ==1){
        //ambient
        vec3 ambient =  light.ambient;
        if (material.type ==0){
            ambient = ambient * material.ambient;
        } else {
            ambient = ambient* img.rgb * material.ambient;
        }

        //diffuse
        vec3 normal = normalize(vNormal);
        vec3 lightDir = normalize(light.pos - vFragPos);
        float diff = max(dot(normal, lightDir), 0.0);
        vec3 diffuse = diff * light.diffuse;

        if (material.type ==0){
            diffuse = diffuse * material.diffuse;
        } else {
            diffuse = img.rgb * diffuse * material.diffuse;
        }

        //specular
        vec3 viewDir = normalize(viewPos - vFragPos);
        vec3 reflectDir = reflect(-lightDir, normal);
        float spec =  max(dot(viewDir, reflectDir), 0.0);

        float specpow =0.0;
        if (material.shininess >0.0){
            specpow = pow(spec, material.shininess);
        } else {
            specpow = spec;
        }
        vec3 specular = specpow * light.specular * material.specular;
        //all
        gl_FragColor = vec4(ambient + diffuse + specular, alpha);
        if (maskType == 1){
            vec4 mask_color = texture2D(mask, vCoordinate.xy);
            gl_FragColor.a = mask_color.a;
        }
    }else{
        if (maskType == 1){
            vec4 mask_color = texture2D(mask, vCoordinate.xy);
            gl_FragColor.a = mask_color.a * alpha;
        }else{
            gl_FragColor.rgb = img.rgb;
            gl_FragColor.a = img.a * alpha;
        }
    }

}



