#extension GL_OES_EGL_image_external:require
precision mediump float;
varying vec2 textureCoordinate;
uniform samplerExternalOES inputImageTexture;
uniform float u_filterRatioV;
uniform float texelWidthS;
uniform float texelHeightS;
uniform float sigma_c;
uniform float sigma_s;
vec4 clampForColor(vec4 color) {
    return clamp(color, vec4(0.0), vec4(1.0));
}
vec3 rgb2yuv(vec3 rgb) { float R = rgb.r;
    float G = rgb.g;
    float B = rgb.b;
    float Y = 0.256788 * R + 0.504129 * G + 0.097906 * B + 16.0 / 256.0;
    float U = -0.148223 * R - 0.290993 * G + 0.439216 * B + 0.5;
    float V = 0.439216 * R - 0.367788 * G - 0.071427 * B + 0.5;
    vec3 resColor = clamp(vec3(Y, U, V), 0.0, 1.0);
    return resColor;
}
 vec3 yuv2rgb(vec3 yuv) {
    float C = yuv.r - 16.0 / 256.0;
    float D = yuv.g - 0.5;
    float E = yuv.b - 0.5;
    float R = 1.164383 * C + 1.596027 * E;
    float G = 1.164383 * C - (0.391762 * D) - (0.812968 * E);
    float B = 1.164383 * C + 2.017232 * D;
    vec3 resColor = clamp(vec3(R, G, B), 0.0, 1.0);
    return resColor;
 }
 float brightEnhance(float brightVal, float brightScale) {
    float brightVal_update = brightVal * (brightScale - 1.0) + 1.0;
    return log(brightVal_update) / log(brightScale);
 }
void main() {
    float constVal1 = 2.*sigma_c*sigma_c;
    float constVal2 = 2.*sigma_s*sigma_s;
    float sf = 0.0;
    vec3 sg = vec3(0., 0., 0.);
    vec3 after_filter;
    const int unit = 1;
    float wg;
    float g;
    float s;
    float f;
    int i;
    float possible;
    float sumdistance = 0.0;
    float edgeflag = 27.0/float(unit);
    vec3 irgb = texture2D(inputImageTexture, textureCoordinate).rgb;
    for (i = -unit;i <= unit;i++) {
        vec3 currgb = texture2D(inputImageTexture, textureCoordinate + vec2(i)*vec2(texelWidthS, texelHeightS)).rgb;
        wg = exp(-float(i*i)/constVal1);
        g = distance(currgb, irgb);
        s = exp(-g*g/constVal2);
        sumdistance+= g*g;
        f = s*wg;
        sf += f;
        sg += f*currgb;
    }
    after_filter = sg/sf;
    possible = 1.0 - clamp(sumdistance*edgeflag, 0.0,1.0);
    vec4 biliteralRes = vec4(mix(irgb, after_filter, possible), 1.0);
    vec3 yuvFromRgb = rgb2yuv(biliteralRes.rgb);
    yuvFromRgb.r = brightEnhance(yuvFromRgb.r, 2.0);
    biliteralRes.rgb = yuv2rgb(yuvFromRgb);
    gl_FragColor.rgb = mix(irgb.rgb, biliteralRes.rgb, u_filterRatioV);
    gl_FragColor.a = 1.0;
    //gl_FragColor = vec4(u_filterRatioV, gl_FragColor.g,0,1.);
}