 #extension GL_OES_EGL_image_external : require
varying highp vec2 textureCoordinate;
uniform samplerExternalOES inputImageTexture;
uniform lowp float brightness;
uniform lowp float filterRatio;
const lowp vec3 W = vec3(0.2125, 0.7154, 0.0721);
lowp vec3 rgb2yuv(lowp vec3 rgb) {
    lowp float R = rgb.r;
    lowp float G = rgb.g;
    lowp float B = rgb.b;
    lowp float Y = 0.256788 * R + 0.504129 * G + 0.097906 * B + 16.0 / 256.0;
    lowp float U = -0.148223 * R - 0.290993 * G + 0.439216 * B + 0.5;
    lowp float V = 0.439216 * R - 0.367788 * G - 0.071427 * B + 0.5;
    lowp vec3 resColor = vec3(Y, U, V); return resColor;
}

lowp vec3 yuv2rgb(lowp vec3 yuv) {
    lowp float C = yuv.r - 16.0 / 256.0;
    lowp float D = yuv.g - 0.5;
    lowp float E = yuv.b - 0.5;
    lowp float R = 1.164383 * C + 1.596027 * E;
    lowp float G = 1.164383 * C - (0.391762 * D) - (0.812968 * E);
    lowp float B = 1.164383 * C + 2.017232 * D;
    lowp vec3 resColor = vec3(R, G, B); return resColor;
}
lowp float brightEnhance(lowp float brightVal,lowp float brightScale) {
    lowp float brightVal_update = brightVal * (brightScale - 1.0) + 1.0;
    return log(brightVal_update) / log(brightScale);
}

lowp vec3 BrightnessContrastSaturation(lowp vec3 color, lowp float brt, lowp float con, lowp float sat) {
    lowp vec3 black = vec3(0., 0., 0.);
    lowp vec3 middle = vec3(0.5, 0.5, 0.5);
    lowp float luminance = dot(color, W);
    lowp vec3 gray = vec3(luminance, luminance, luminance);
    lowp vec3 brtColor = mix(black, color, brt);
    lowp vec3 conColor = mix(middle, brtColor, con);
    lowp vec3 satColor = mix(gray, conColor, sat);
    return satColor;
}
lowp vec3 Sharpen() {
    int kernelSize = 5;
    int offsetSize = 2;
    lowp vec2 stOffset[25];
    lowp float stSample[25];
    lowp vec3 target = vec3(0., 0., 0.);
    lowp float ResS = 120.;
    lowp float ResT = 120.;
    int idx = 0;
    for(lowp float x = -2.; x<=2.5; x+=1.){
     for(lowp float y = -2.; y<=2.5; y+=1.){
        stOffset[idx++] = vec2( textureCoordinate.x + x/ResS, textureCoordinate.y + y/ResT );
      }
    }
    int K = 25;
    for(int i = 0; i<K; i++) stSample[i] = dot(texture2D(inputImageTexture, stOffset[i]).rgb, W);
        lowp float mag = stSample[12];
        for(int i = 0; i<K; i++){
            if(i != 12) mag -= stSample[i]/25.;
         }
    target = vec3(mag, mag, mag); return target;
}

lowp vec3 skin() {
    lowp vec3 target;
    lowp vec3 irgb = texture2D(inputImageTexture, textureCoordinate).rgb;
    lowp float L = dot(irgb, W);
    lowp float ResS = 120.;
    lowp float ResT = 120.; int K = 9;
    lowp vec2 stp0 = vec2(1./ResS, 0.);
    lowp vec2 st0p = vec2(0., 1./ResT);
    lowp vec2 stpp = vec2(1./ResS, 1./ResT);
    lowp vec2 stpm = vec2(1./ResS, -1./ResT);
    lowp vec2 stOffset[9];
    stOffset[0] = textureCoordinate-stpp;
    stOffset[1] = textureCoordinate-stp0;
    stOffset[2] = textureCoordinate-stpm;
    stOffset[3] = textureCoordinate-st0p;
    stOffset[4] = textureCoordinate;
    stOffset[5] = textureCoordinate+st0p;
    stOffset[6] = textureCoordinate+stpm;
    stOffset[7] = textureCoordinate+stp0;
    stOffset[8] = textureCoordinate+stpp;
    lowp float sigma_c = 30.;
    lowp float sigma_s = .03;
    lowp vec3 sf = vec3(0., 0., 0.);
    lowp vec3 sg = vec3(0., 0., 0.);
    target = vec3(0., 0., 0.);
    for(int i = 0; i<K; i++){
        lowp float wg = exp(-(stOffset[i].x*stOffset[i].x+stOffset[i].y*stOffset[i].y)/(2.*sigma_c*sigma_c));
        lowp vec3 g = texture2D(inputImageTexture, stOffset[i]).rgb*wg;
        lowp vec3 s = exp(-(irgb-g)*(irgb-g)/(2.*sigma_s*sigma_s));
        lowp vec3 f = s*wg; sf += f; sg += f*g;
     }
    target = sg/sf; return target;
}
lowp vec3 CreateCircleLightSource(){
    lowp vec3 color; lowp float rx = .5;
    lowp float ry = .5;
    lowp float r = 1.;
    lowp float rr = r*r;
    lowp float dd = (textureCoordinate.x-rx)*(textureCoordinate.x-rx)+(textureCoordinate.y-ry)*(textureCoordinate.y-ry);
    lowp float d = sqrt(dd);
    if( d<=r ){
        lowp float dc = d/2. ; color = vec3(dc, dc, dc);
    } else{
        lowp float dc = d; color = vec3(dc, dc, dc);
    }
     return color;
}
void main() {
    lowp vec3 irgb = texture2D(inputImageTexture, textureCoordinate).rgb;
    lowp vec3 originalColor = irgb;
    lowp float beta = 3.1;
    lowp float T_bright = 1.0 + beta*0.01;
    lowp float T_contrast = 1.0 + beta*0.1;
    lowp float T_saturation = 1.0 + beta*0.03;
    lowp vec3 bcs_result = BrightnessContrastSaturation(irgb, T_bright, T_contrast, T_saturation);
    bcs_result = vec3(bcs_result.r*(1.0 + beta*0.02), bcs_result.g*1., bcs_result.b*1.);
    lowp vec3 after_filter = bcs_result - CreateCircleLightSource()/5.;
    lowp vec3 yuvFromRgb = rgb2yuv(after_filter); yuvFromRgb.r = brightEnhance(yuvFromRgb.r, 2.0);
    after_filter = yuv2rgb(yuvFromRgb);
    after_filter = mix(originalColor, after_filter, filterRatio);
    gl_FragColor = vec4(after_filter, 1.);
    //gl_FragColor = vec4(filterRatio, 0,after_filter.b,1.);
}